import logging
import coloredlogs

Logger = logging.getLogger('Magneto')

# Initialize coloredlogs.
coloredlogs.install(level=logging.DEBUG,
                    show_timestamps=False,
                    show_hostname=False,
                    show_name=False)



#!/usr/bin/env python
import click
import logging
import os
import pytest

from .logger import Logger
from .base import BaseTestCase
from .magneto import Magneto
from .utils import ADB, wait_for_device, unlock_device


class MagnetoPlugin(object):
    def pytest_addoption(self, parser):
        parser.addoption('--app-package')
        parser.addoption('--app-activity')
        parser.addoption('--apk-path', help='APK to install on device before tests')
        parser.addoption('--clean-install', default=False, action='store_true',
                         help='Uninstalls apk before installation. Requires --apk-path.')
        parser.addoption('--device-id', help='Device id to run tests on')
        parser.addoption('--save-data-on-failure', default=False, action='store_true',
                         help='Save hierarchy.xml/screenshot.png/logcat.log if a test fails')
        parser.addoption('--include-video-on-failure', default=False, action='store_true',
                         help='Wether to record video or not')
        parser.addoption('--magneto-failed-data-dir', default='/tmp/magneto_test_data', help='')
        parser.addoption('--wait-for-element-timeout', default=5000, type='int',
                         help='wait_for_element() default timeout')

    def pytest_configure(self, config):
        apk_path = config.getoption('--apk-path')
        app_package = config.getoption('--app-package')
        app_activity = config.getoption('--app-activity')
        device_id = config.getoption('--device-id')
        clean_install = config.getoption('--clean-install')

        Magneto.configure(device_id)

        wait_for_device()
        unlock_device()

        if apk_path:
            if not apk_path.startswith('/'):
                apk_path = os.path.abspath(os.path.join(os.getcwd(), apk_path))

            if clean_install:
                ADB.uninstall(app_package)
            apk_installed = ADB.install(apk_path, '-r')

            if not apk_installed:
                raise RuntimeError('Could not install apk.')

        # launch app
        ADB.exec_cmd('shell am start {0}/{1}'.format(app_package, app_activity)).wait()

    def pytest_unconfigure(self, config):
        BaseTestCase.unconfigure(config)

    def pytest_runtest_makereport(self, item, call, __multicall__):
        """
        Skip remaining tests if current_test (item) is blocker and it has failed.
        """
        # get current report status from _pytest.runner.pytest_runtest_makereport
        report = __multicall__.execute()
        getattr(BaseTestCase, 'pytest_runtest_' + report.when)(item, report)
        return report


@click.group()
@click.option('--log', default='INFO', help='logging level. default:INFO')
def main(log):
    numeric_level = getattr(logging, log.upper(), None)
    Logger.setLevel(numeric_level)


@main.command(context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True,
))
@click.argument('tests_path', default='.')
@click.pass_context
def run(ctx, tests_path):
    pytest.main(['-sv', tests_path] + ctx.args, plugins=[MagnetoPlugin()])


@main.command()
@click.argument('app', default='no_app')
def init(app):
    from .utils import Bootstrap

    Bootstrap(app)


if __name__ == '__main__':
    main()



import re
from ..magneto import Magneto


class Assert(object):
    @classmethod
    def current_package(cls, *expected_packages, **kwargs):
        """
        Checks current package equals the expected.
        Raises exception if timed out

        :param expected_packages: package names
        :param str msg: Overrides default exception message

        Example::

            # single package
            Assert.current_package('com.android.chrome')

            # any package
            Assert.current_package('com.google.android.gm', 'com.android.email')
        """
        magneto = Magneto.instance()

        found = magneto.wait_for_true(lambda: magneto.info['currentPackageName'] in expected_packages, **kwargs)

        if not found:
            msg = kwargs.get('msg', None)
            current_package = magneto.info['currentPackageName']
            default_msg = "{0} is the current package, not {1} ".format(current_package, ' nor '.join(expected_packages))

            raise AssertionError(cls._format_message(msg, default_msg))

    @classmethod
    def true(cls, expr, msg=None):
        """
        Checks that the expression is true

        Assert that the device is a tablet::

            Assert.true(device.type == 'tablet')
        """
        if not expr:
            msg = cls._format_message(msg, "{} is not true".format(expr))
            raise AssertionError(msg)

    @classmethod
    def false(cls, expr, msg=None):

        """
        Checks that the expression is false

        Assert that the device is not a tablet::

            Assert.false(device.type == 'tablet')
        """
        if expr:
            msg = cls._format_message(msg, "{} is not false".format(expr))
            raise AssertionError(msg)

    @classmethod
    def regexp_matches(cls, text, expected_regexp, msg=None):
        """
        Fail the test unless the text matches the regular expression.

        Assert that the user agent string matches a OS 4.1.2::

            Assert.regexp_matches(user_agent, '.*\s4.1.2;')

        """
        # compile if not already
        if isinstance(expected_regexp, basestring):
            expected_regexp = re.compile(expected_regexp)

        # test regex
        if not expected_regexp.search(text):
            msg = msg or "Regexp mismatch"
            msg = '{0}: {1} not found in {2}'.format(msg, expected_regexp.pattern, text)
            raise AssertionError(msg)

    @classmethod
    def _format_message(cls, msg, default_msg):
        return msg or default_msg



import os
import subprocess
import re
import threading
from functools import wraps
from concurrent.futures import Future, wait
import time

from ..logger import Logger


class ADB(object):
    """
    Wrapper for adb commands.

    Import::

        from magneto.utils.adb import ADB

    :required: Android SDK installed and ANDROID_HOME pointing to SDK directory location.
    """

    _lock = threading.Lock()
    device_id = None
    ADB_PATH = os.path.join(os.environ['ANDROID_HOME'], 'platform-tools', 'adb')

    @classmethod
    def exec_cmd(cls, exec_cmd, stdin=None, stdout=None, stderr=None):
        """
        Executes adb commands::

            ADB.exec_cmd('shell uninstall com.android.package')

        In order to retrieve the process output, tap into the stdout::

            import subprocess

            proc = ADB.exec_cmd('shell pm list packages', stdout=subprocess.PIPE)
            list = proc.stdout.readline().strip('\\r\\n')

        :param str exec_cmd: Adb command to execute
        :param stdin:
        :param stdout:
        :param stderr:
        :return: Adb process
        """
        with cls._lock:
            cmd = 'exec {adb_path} {device_id} {command}'.format(
                adb_path=cls.ADB_PATH,
                device_id='-s {0}'.format(cls.device_id) if cls.device_id else '',
                command=exec_cmd
            )

            return subprocess.Popen(cmd, shell=True, stdin=stdin, stdout=stdout, stderr=stderr)

    @classmethod
    def get_log(cls):
        """
        Get log adb logcat.

        :return: Log dump
        """
        dump = []
        p = cls.exec_cmd("logcat -v time -d", stdout=subprocess.PIPE)
        for l in p.stdout:
            dump.append(l.strip())

        return dump

    @classmethod
    def clear_log(cls):
        """
        Clears adb log

        :return: Adb process
        """
        return cls.exec_cmd("logcat -c")

    @classmethod
    def start_activity(cls, package_name, activity_name, extras=None):
        """
        Start app activity.

        Open Gmail app::

            ADB.start_activity('com.google.android.gm', 'com.google.android.gm.ConversationListActivity')

        :param str package_name: Package name
        :param str activity_name: Activity name
        :return:
        """
        Logger.debug('opening activity {}'.format(activity_name))
        cls.exec_cmd("shell am start {0} {1}/{2}".format(extras or '', package_name, activity_name)).wait()

    @classmethod
    def kill_process(cls, package_name):
        """
        Stops given app::

            ADB.kill_process('com.android.chrome')

        :param str package_name: Package name to stop
        :return:
        """
        cls.exec_cmd("shell am force-stop {0}".format(package_name)).wait()

    @classmethod
    def uninstall(cls, package_name):
        Logger.debug('Uninstalling app {}'.format(package_name))
        cls.exec_cmd('uninstall {0}'.format(package_name)).wait()

        # remove app cache
        cls.exec_cmd('shell su -c "rm -rf /data/data/{0}"'.format(package_name)).wait()

    @classmethod
    def install(cls, apk_path, extra_params='', retry=True):
        Logger.debug('Installing app {}'.format(apk_path))
        p = cls.exec_cmd('install {} {}'.format(extra_params, apk_path), stdout=subprocess.PIPE)
        result = p.communicate()[0].strip().split('\r\n')[-1]

        if result == 'Success':
            Logger.debug('App installed')
            return True
        elif result != 'Success' and retry:
            Logger.debug('App install failed: {}'.format(result))
            Logger.debug('Retrying install...')
            return cls.install(apk_path, extra_params=extra_params, retry=False)
        else:
            Logger.debug('App install failed: {}'.format(result))
            return False

    @classmethod
    def getprop(cls, prop):
        p = cls.exec_cmd('shell getprop {}'.format(prop), stdout=subprocess.PIPE)
        return p.stdout.readline().strip('\r\n')

    @classmethod
    def set_datetime(cls, dt):
        adb_date = dt.strftime('%Y%m%d.%H%M%S')
        cls.exec_cmd("shell \"su -c 'date -s {adb_date}'\"".format(adb_date=adb_date)).wait()


class RegexMatcher(object):
    def __init__(self, pattern):
        self._pattern = pattern

    def __call__(self, line):
        return self._pattern.search(line)

    def __str__(self):
        return self._pattern.pattern[:20]+'...'


class ADBLogWatch(threading.Thread):
    """
    Enables watching adb logcat logs and asserting that they certain ones appeared::

            from magneto.utils.adb import ADBLogWatch

            def test_button_click_log(self, watcher):
                with ADBLogWatch() as watcher:
                    watcher.watch('button clicked')
                    self.magneto(text='Click here').click()
                    watcher.assert_done()
    """
    instance = None

    def __init__(self):
        ADB.clear_log().wait()
        super(ADBLogWatch, self).__init__()
        self._watchers = {}
        self.asked_to_stop = False

        if self.__class__.instance is not None:
            raise RuntimeError('Only one instance of {} is allowed at any time.'.format(self.__class__.__name__))

        self.__class__.instance = self

    @classmethod
    def wrap(cls, fn):
        @wraps(fn)
        def wrapper(instance, *args, **kwargs):
            self = cls()
            with self:
                return fn(instance, self, *args, **kwargs)
        return wrapper

    def __enter__(self, *_):
        self.start()
        return self

    def __exit__(self, *_):
        self.exit()

    def run(self):
        p = ADB.exec_cmd('logcat', stdout=subprocess.PIPE)
        while not self.asked_to_stop:
            line = p.stdout.readline().strip()
            if line == '':
                break
            for matcher, (future, min_times) in self._watchers.items():
                match = matcher(line)
                if match:
                    Logger.debug('Found logcat for "{}"'.format(matcher))
                    if min_times == 1:
                        future.set_result(line)
                        Logger.debug('Removing watch "{}"'.format(matcher))
                        del self._watchers[matcher]
                    else:
                        self._watchers[matcher] = future, min_times - 1

        p.terminate()
        Logger.debug('ADB logcat process terminated')

    def exit(self):
        self.__class__.instance = None
        self.asked_to_stop = True
        self.join()

    def watch(self, pattern, **kwargs):
        """
        Watch for given pattern in logs.

        :param pattern: Regular expression pattern to perform on logcat lines
        :param int min_times: Minimum times a certain pattern should appear in log
        """
        if callable(pattern):
            Logger.debug('watching pattern "{}"'.format(str(pattern)))
            future = Future()
            self._watchers[pattern] = future, kwargs.get('min_times', 1)

            return future
        else:
            return self.watch_compiled(re.compile(pattern), **kwargs)

    def watch_compiled(self, pattern, min_times=1):
        regex_matcher = RegexMatcher(pattern)
        Logger.debug('watching pattern "{}"'.format(str(regex_matcher)))
        future = Future()
        self._watchers[regex_matcher] = future, min_times

        return future

    def assert_done(self, timeout=15, stall=None, futures=None):
        """
        Asserts if all/some watches exist in log.

        :param futures: The futures we want to wait to end
        :param timeout: Amount of time in seconds to wait for watches to appear in log. Defaults to 5 seconds.
        :param stall: Amount of time in seconds to keep waiting before exiting the wait. More info in https://github.com/EverythingMe/magneto/issues/1

        Example::

            from magneto.utils.adb import ADBLogWatch

            def test_incoming_call(self, watcher):
                with ADBLogWatch() as watcher:
                    watcher.watch('call arrived')
                    call_device()
                    watcher.assert_done()

        Example with stall::

            from magneto.utils.adb import ADBLogWatch

            def test_incoming_call(self, watcher):
                with ADBLogWatch() as watcher:
                    watcher.watch('call arrived')
                    call_device()
                    watcher.assert_done(timeout=10, stall=120)
        """

        if futures:
            if not isinstance(futures, list):
                futures = [futures]
        else:
            futures = [future for (future, _) in self._watchers.itervalues()]

        Logger.debug('waiting up to {} seconds for {} watchers'.format(timeout, len(futures)))

        start_time = time.time()
        done, pending = wait(futures, timeout=timeout)
        if not pending:
            Logger.debug('watchers done')
            return

        if stall:
            stall_time = stall-int(time.time() - start_time)
            Logger.debug('Stalling for another %d seconds' % stall_time)
            done, pending = wait(futures, timeout=stall_time)
            if not pending:
                Logger.debug('watchers done')

        patterns_left = '\n'.join('pattern: {}'.format(str(r)) for r in self._watchers)
        raise AssertionError(
            'assert_done failure.\nWaited {} seconds but still have {} watchers:\n{}'
            .format(timeout, len(self._watchers), patterns_left)
        )

    def assert_watch(self, *args, **kwargs):
        """
        Asserts specific watchers if they exists in log

        :param args: <Future> all watchers we want to wait to end
        :param kwargs: extra params to pass to assert_done like timeout..
        :return: self.asset_done
        """
        return self.assert_done(futures=list(args), **kwargs)


class ADBVideoCapture(threading.Thread):
    def __init__(self, video_file_name, timelimit=180, bitrate=1000000, size='1280x720'):
        super(ADBVideoCapture, self).__init__()

        self.video_file_name = video_file_name
        self.recording_process = None
        self.files = []
        self.asked_to_stop = False
        self.timelimit = timelimit
        self.bitrate = bitrate
        self.size = size

        self.daemon = True
        self.start()

    def run(self):
        self.start_recording()
        while not self.asked_to_stop:
            if self.recording_process.poll() is not None:
                self.start_recording()
        self.recording_process.terminate()

    def start_recording(self):
        self.files.append('/sdcard/{0}-{1}.video.mp4'.format(self.video_file_name, len(self.files) + 1))
        self.recording_process = ADB.exec_cmd(
            'shell screenrecord --bit-rate {bitrate} --time-limit {timelimit} --size {size} {file}'.format(
                bitrate=self.bitrate,
                timelimit=self.timelimit,
                size=self.size,
                file=self.files[
                    len(self.files) - 1]))

    def stop_recording(self):
        self.asked_to_stop = True

    def save_files(self, save_file_path):
        for video_file in self.files:
            ADB.exec_cmd('pull {0} {1}'.format(video_file, save_file_path)).wait()

    def delete_files(self):
        for video_file in self.files:
            ADB.exec_cmd('shell rm {0}'.format(video_file)).wait()



import sys
from IPython import embed

from .magneto import Magneto


m = magneto = None


def stop():
    if m:
        m.server.stop()


def main():
    args = sys.argv[1:]
    Magneto.configure(*args)
    m = magneto = Magneto.instance()

    embed()


if __name__ == '__main__':
    main()


import threading
import time
import itertools

from uiautomator import AutomatorDevice, AutomatorDeviceObject, Selector

from .utils import get_center, Timeout, TimeoutError, get_config
from .utils.adb import ADB


class Magneto(threading.local, AutomatorDevice):
    """
    Inherits from ``uiautomator.device``, which has `docs available <https://github.com/xiaocong/uiautomator/blob/master/README.md>`_.
    """
    _instance = None
    _args = ()
    _kwargs = {}
    _device_id = None

    @classmethod
    def instance(cls):
        if cls._instance is None:
            cls._instance = cls(cls._device_id, *cls._args, **cls._kwargs)
        return cls._instance

    @classmethod
    def configure(cls, device_id=None, *args, **kwargs):
        cls._device_id = device_id

        ADB.device_id = cls._device_id
        cls._manufacturer = ADB.getprop('ro.product.manufacturer').lower()
        cls._args = args
        cls._kwargs = kwargs

    def __call__(self, *args, **kwargs):
        el = MagnetoDeviceObject(self, Selector(**kwargs))
        if not el.exists:
            self.server.stop()
            self.server.start()
            el = MagnetoDeviceObject(self, Selector(**kwargs))
        return el

    @property
    def info(self):
        info = dict(super(Magneto, self).info, manufacturer=self._manufacturer)
        return info

    def drag(self, *args, **kwargs):
        """
        Allows dragging elements on screen by element or coords.

        :param args:
        :param kwargs:

        **Usage**:

        Drag element to element::

            source_el = self.magneto(resourceId=ids.foo)
            target_el = self.magneto(resourceId=ids.bar)

            self.magneto.drag(source_el, target_el)

        Drag element to coordinate::

            source_el = self.magneto(resourceId=ids.foo)
            target_x = 200
            target_y = 100

            self.magneto.drag(source_el, target_x, target_y)

        Drag coordinate to element::

            source_x = 200
            source_y = 100
            target_el = self.magneto(resourceId=ids.foo)

            self.magneto.drag(source_x, source_y, target_el)

        Drag coordinate to coordinate::

            source_x = 200
            source_y = 100
            target_x = 200
            target_y = 100

            self.magneto.drag(source_x, source_y, target_x, target_y)
        """
        if len(args) == 2:
            (start_x, start_y) = args[0].center()
            (end_x, end_y) = args[1].center()
        elif len(args) == 3:
            if args[0].__class__ != 'int':
                (start_x, start_y) = args[0].center()
                (end_x, end_y) = args[1:]
            else:
                (start_x, start_y) = args[:2]
                (end_x, end_y) = args[2].center()
        elif len(args) == 4:
            (start_x, start_y, end_x, end_y) = args

        return super(Magneto, self).drag(start_x, start_y, end_x, end_y, **kwargs)

    def wait_for_element(self, timeout=None, **kwargs):
        """
        Wait for an element to show up::

            self.magneto.wait_for_element(resourceId=ids.foo)

        :param int timeout: Timeout in ms. Defaults to --wait-for-element-timeout which is 5000 by default.
        :param kwargs:
        :return element el:
        """
        if not timeout:
            timeout = get_config('--wait-for-element-timeout')

        el = self(**kwargs)
        el.wait.exists(timeout=timeout)
        return el

    def wait_for_true(self, function, timeout=15000, **kwargs):
        """
        Waits for given function to return True::

            self.magneto.wait_for(lambda: self.magneto.info['displayRotation'] == 90)

        :param function function:
        :param int timeout: Timeout in ms. Default to ``15000``
        :return Boolean result: The result of the last function invocation
        """
        result = False

        try:
            with Timeout(seconds=timeout/1000):
                while not result:
                    result = function(**kwargs)
                    time.sleep(0.5)

        except TimeoutError:
            pass

        return result

    def get_element_children(self, el, **kwargs):
        """
        Yields specific element children
        :param element el:

        :return generator
        """
        for i, _ in enumerate(itertools.repeat(None)):
            selector = el.child(instance=i, **kwargs)
            if not selector.exists:
                break
            yield selector


class MagnetoDeviceObject(AutomatorDeviceObject):
    def __init__(self, device, selector):
        super(MagnetoDeviceObject, self).__init__(device, selector)

    def center(self):
        return get_center(self.info['visibleBounds'])

    def child(self, **kwargs):
        """set childSelector."""
        return MagnetoDeviceObject(
            self.device,
            self.selector.clone().child(**kwargs)
        )

    def sibling(self, **kwargs):
        """set fromParent selector."""
        return MagnetoDeviceObject(
            self.device,
            self.selector.clone().sibling(**kwargs)
        )


class MagnetoException(Exception):
    pass


from __future__ import absolute_import
from contextlib import contextmanager
from multiprocessing import TimeoutError
import signal
import datetime
import os
import subprocess
import time
import urllib
import zipfile
import shutil

import pytest

from .adb import ADB
from ..logger import Logger


def get_center(bounds):
    """
    Returns given element center coords::

        from magneto.utils import get_center

        element = self.magneto(text='Foo')
        (x, y) = get_center(element.info['bounds'])

    :param dict bounds: Element position coordinates (top, right, bottom, left)
    :return: x and y coordinates of element center
    """

    x = bounds['right'] - ((bounds['right'] - bounds['left']) / 2)
    y = bounds['bottom'] - ((bounds['bottom'] - bounds['top']) / 2)
    return x, y


def get_config(attr, default=None):
    """
    Allows access to config parameters::

        from magneto.utils import get_config

        package = get_config('--app-package')

    :param str attr: Command line argument
    :return: Requested config value
    """

    # must have this check to avoid sphinx-autodoc exception
    if getattr(pytest, 'config', None) != None:
        return pytest.config.getoption(attr) or default
    else:
        return default


@contextmanager
def timewarp(timedelta_):
    now = datetime.datetime.now()
    future = now + timedelta_
    ADB.set_datetime(future)

    try:
        yield
    finally:
        now = datetime.datetime.now()
        ADB.set_datetime(now)


class Timeout():
    """
    Allows polling a function till success or timeout::

        import time
        from magneto.utils import Timeout

        result = False

        with Timeout(seconds=5):
            while not result:
                result = some_function()
                time.sleep(0.5)


    :param integer seconds: Timeout value in seconds. Defaults to 1.
    :param str error_message: Error message to display when timeout occurs. Defaults to 'Timeout'.
    """

    def __init__(self, seconds=1, error_message='Timeout'):
        self.seconds = seconds or 1
        self.error_message = error_message

    def handle_timeout(self, signum, frame):
        Logger.debug('Timeout reached {} seconds limit'.format(self.seconds))
        raise TimeoutError(self.error_message)

    def __enter__(self):
        Logger.debug('Timeout started for {} seconds'.format(self.seconds))
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.seconds)

    def __exit__(self, type, value, traceback):
        Logger.debug('Timeout stopped.')
        signal.alarm(0)


def unlock_device():
    """
    Powers on device and unlocks it.
    """

    # read device screen state
    p = ADB.exec_cmd("shell 'if [ -z $(dumpsys power | grep mScreenOn=true) ]; then echo off; else echo on;fi'",
                     stdout=subprocess.PIPE)
    device_screen = p.stdout.readline().strip('\r\n')

    if device_screen == 'off':
        # power on device
        ADB.exec_cmd('shell input keyevent 26').wait()

    # unlock device
    ADB.exec_cmd('shell input keyevent 82').wait()


def wait_for_device():
    """
    Wait for device to boot. 1 minute timeout.
    """

    wait_for_device_cmd = 'wait-for-device shell getprop sys.boot_completed'
    p = ADB.exec_cmd(wait_for_device_cmd, stdout=subprocess.PIPE)
    boot_completed = p.stdout.readline().strip('\r\n')

    try:
        with Timeout(seconds=60):
            while boot_completed != '1':
                time.sleep(1)
                p = ADB.exec_cmd(wait_for_device_cmd, stdout=subprocess.PIPE)
                boot_completed = p.stdout.readline().strip('\r\n')
                Logger.debug('Waiting for device to finish booting (adb shell getprop sys.boot_completed)')
    except TimeoutError:
        Logger.debug('Timed out while waiting for sys.boot_completed, there might not be a default launcher set, trying to run anyway')
        pass


class Bootstrap(object):
    _map = {
        'no_app': 'https://github.com/EverythingMe/magneto-init/archive/master.zip',
        'calc': 'https://github.com/EverythingMe/magneto-demo-calc/archive/master.zip'
    }

    def __init__(self, name):
        if name not in self._map:
            raise Exception('{} not recognized'.format(name))

        filename, headers = urllib.urlretrieve(self._map[name])

        with zipfile.ZipFile(filename) as zip_file:
            rootdir = zip_file.namelist()[0]
            for member in zip_file.namelist()[1:]:
                if not os.path.basename(member):
                    # create dir from zipfile
                    os.mkdir(os.path.join(os.path.curdir, member.replace(rootdir, '')))
                else:
                    # copy file (taken from zipfile's extract)
                    source = zip_file.open(member)
                    target = file(os.path.join(os.path.curdir, member.replace(rootdir, '')), "wb")
                    with source, target:
                        shutil.copyfileobj(source, target)



from __future__ import absolute_import

import os
from datetime import datetime

import pytest

from .magneto import Magneto
from .logger import Logger
from .utils import get_config
from .utils.adb import ADB, ADBVideoCapture


class BaseTestCase(object):
    """
    Used as the test case base class.

    Example::

        class FooTestCase(BaseTestCase):
            '''
            Test Foo
            '''

            def test_bar(self):
                '''
                Test bar
                '''
                self.assertTrue(True)
    """

    magneto = None
    blocker_failed = False
    video_thread = None
    current_test = None
    test_suite_timestamp = None
    test_number = 0

    @classmethod
    def setup_class(cls):
        if BaseTestCase.magneto is None:
            BaseTestCase.magneto = Magneto.instance()
            BaseTestCase.test_suite_timestamp = datetime.now().strftime("%Y%m%d%H%M")

    @classmethod
    def teardown_class(cls):
        pass

    @classmethod
    def unconfigure(cls, *_):
        if BaseTestCase.magneto:
            if not BaseTestCase.blocker_failed:
                BaseTestCase.magneto.press.home()
            BaseTestCase.magneto.server.stop()
            BaseTestCase.magneto = None
            BaseTestCase.video_thread = None
            BaseTestCase.current_test = None
            BaseTestCase.test_number = 0
            BaseTestCase.test_suite_timestamp = None

    @classmethod
    def pytest_runtest_setup(cls, item, report):
        cls.test_number += 1
        cls.current_test = '{number}-{name}'.format(number=cls.test_number, name=item.name)
        if not report.skipped and get_config('--save-data-on-failure') and get_config('--include-video-on-failure'):
            cls.video_thread = ADBVideoCapture(cls.current_test)

    @classmethod
    def pytest_runtest_call(cls, item, report):
        if cls.video_thread:
            cls.video_thread.stop_recording()
        if report.failed:
            if 'blocker' in item.keywords:
                cls.blocker_failed = True
                for test in item.session.items:
                    if test.name != item.name and test._request:
                        test._request.applymarker(
                            pytest.mark.skipif(True,
                                               reason="{0} blocker failed, skipping remaining tests.".format(
                                                   item.name)))

            if get_config('--save-data-on-failure'):
                data_path_suffix = get_config('--device-id', 'device') + '-' + cls.test_suite_timestamp
                data_path = os.path.join(get_config('--magneto-failed-data-dir'), data_path_suffix)
                Logger.debug('Saving failed test data into {}'.format(data_path))
                if not os.path.isdir(data_path):
                    Logger.debug('Creating directory {}'.format(data_path))
                    os.makedirs(data_path)

                Logger.debug('Saving test logcat')
                logcat_file_path = os.path.join(data_path, '{0}.logcat.log'.format(cls.current_test))
                logcat_buffer = ADB.get_log()
                logcat_file = open(logcat_file_path, 'w')
                logcat_file.write('\n'.join(logcat_buffer))
                logcat_file.close()

                Logger.debug('Saving test hierarchy xml')
                cls.magneto.dump(os.path.join(data_path, '{0}.hierarchy.uix'.format(cls.current_test)))
                Logger.debug('Saving test screenshot')
                cls.magneto.screenshot(os.path.join(data_path, '{0}.screenshot.png'.format(cls.current_test)))
                if cls.video_thread:
                    Logger.debug('Saving test video')
                    cls.video_thread.save_files(data_path)

    @classmethod
    def pytest_runtest_teardown(cls, *_):
        ADB.clear_log()
        if cls.video_thread:
            cls.video_thread.delete_files()
            cls.video_thread = None

    def setup_method(self, method):
        pass

    def teardown_method(self, method):
        pass



from setuptools import setup, find_packages

setup(
    name='magneto',
    version='0.31.1',
    description='Magneto - Test Automation for Android',
    author='EverythingMe',
    author_email='automation@everything.me',
    url='http://github.com/EverythingMe/magneto',
    packages=find_packages(),
    install_requires=[
        'pytest',
        'pytest-ordering',
        'uiautomator',
        'futures',
        'coloredlogs',
        'IPython',
        'Click'
    ],

    entry_points={
        'console_scripts': [
            'magneto = magneto.main:main',
            'imagneto = magneto.imagneto:main'
        ]
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing'
    ]
)
